@@ -60,7 +60,7 @@
  *  DispatchEvent - calls appropriate handler for the event
  ************************************************************************/
 void DigestEvent    ( ASEvent *event );
-void afterstep_wait_pipes_input ();
+void afterstep_wait_pipes_input (int timeout_sec);
 
 void
 HandleEvents ()
@@ -88,7 +88,7 @@ HandleEvents ()
             }
             ExecutePendingFunctions();
         }
-        afterstep_wait_pipes_input ();
+        afterstep_wait_pipes_input (0);
         ExecutePendingFunctions();
     }
 }
@@ -121,7 +121,7 @@ LOCAL_DEBUG_OUT( "checking masked events ...%s", "" );
             if( Scr.moveresize_in_progress == NULL )
                 return;
         }
-        afterstep_wait_pipes_input ();
+        afterstep_wait_pipes_input (0);
     }
 }
 
@@ -148,7 +148,7 @@ WaitForButtonsUpLoop ()
 }
 
 Bool
-WaitEventLoop( ASEvent *event, int finish_event_type, long timeout )
+WaitEventLoop( ASEvent *event, int finish_event_type, long timeout, ASHintWindow *hint )
 {
 	unsigned long mask = ButtonPressMask | ButtonReleaseMask |
 						 ExposureMask | KeyPressMask | ButtonMotionMask |
@@ -177,6 +177,8 @@ WaitEventLoop( ASEvent *event, int finish_event_type, long timeout )
 		}
         DigestEvent( event );
         DispatchEvent( event, done );
+		if( event->x.type == MotionNotify && hint ) 
+			update_ashint_geometry( hint, False );
     }
 
     return True ;
@@ -236,6 +238,8 @@ WaitWindowLoop( char *pattern, long timeout )
 	time_t click_end_time = end_time / 4;
 	time_t start_time = time(NULL);
 	ASWindow *asw = NULL ;
+	ASHintWindow *hint ;
+	char *text ;
 
 	end_time += start_time ;
 	click_end_time += start_time ;
@@ -247,6 +251,10 @@ WaitWindowLoop( char *pattern, long timeout )
     if( (asw = complex_pattern2ASWindow( pattern )) != NULL )
   		return asw;
 
+	text = safemalloc(64+strlen(pattern)+1);
+	sprintf( text, "Waiting for window matching \"%s\" ... Press button to cancel.", pattern );
+	hint = create_ashint_window( ASDefaultScr, &(Scr.Look), text);
+	free( text );
 	while (!done)
 	{
 		do
@@ -265,11 +273,15 @@ WaitWindowLoop( char *pattern, long timeout )
 					{
 						end_time = click_end_time ;
 						break;
-					}
-
-                    if( (event.x.type == MapNotify || event.x.type == PropertyNotify )&& event.client )
+					}else if( event.x.type == MotionNotify && hint ) 
+						update_ashint_geometry( hint, False );
+                    
+					if( (event.x.type == MapNotify || event.x.type == PropertyNotify )&& event.client )
                         if( (asw = complex_pattern2ASWindow( pattern )) != NULL )
+						{
+							destroy_ashint_window( &hint );
                             return asw;
+						}
                 }
 			}
 			if( time(NULL) > end_time )
@@ -282,8 +294,9 @@ WaitWindowLoop( char *pattern, long timeout )
 
 		if( time(NULL) > end_time )
 			break;
-		afterstep_wait_pipes_input ();
+		afterstep_wait_pipes_input (1);
 	}
+	destroy_ashint_window( &hint );
     return NULL;
 }
 
@@ -325,19 +338,62 @@ DigestEvent( ASEvent *event )
 {
     register int i ;
     setup_asevent_from_xevent( event );
+    event->client = NULL;
+    SHOW_EVENT_TRACE(event);
     /* in housekeeping mode we handle pointer events only as applied to root window ! */
     if( Scr.moveresize_in_progress && (event->eclass & ASE_POINTER_EVENTS) != 0)
     {
 	    event->context = C_ROOT ;
     	event->widget = Scr.RootCanvas ;
-        event->client = NULL;
         /* we have to do this at all times !!!! */
         if( event->x.type == ButtonRelease && Scr.Windows->pressed )
             release_pressure();
     }else
 	{
-		if( event->w == Scr.Root || 
-			event->w == Scr.ServiceWin ||
+	    XButtonEvent *xbtn = &(event->x.xbutton);
+
+		if( event->w == Scr.Root )
+		{
+			event->context = C_ROOT ;
+#ifndef NO_VIRTUAL
+			if((event->eclass & ASE_MousePressEvent) != 0 && xbtn->subwindow != None )
+			{
+				LOCAL_DEBUG_OUT( "subwindow = %lX", event->x.xbutton.subwindow );
+	    		for( i = 0 ; i < PAN_FRAME_SIDES ; i++ )
+	    		{
+	        		LOCAL_DEBUG_OUT("checking panframe %d, mapped %d", i, Scr.PanFrame[i].isMapped );
+	        		if( Scr.PanFrame[i].isMapped && xbtn->subwindow == Scr.PanFrame[i].win)
+        			{/* we should try and pass through this click onto any client that 
+						maybe under the pan frames */
+						LOCAL_DEBUG_OUT( "looking for client at %+d%+d", xbtn->x_root, xbtn->y_root ); 
+						if( (event->client = find_topmost_client( Scr.CurrentDesk, xbtn->x_root, xbtn->y_root )) != NULL )
+						{
+					    	LOCAL_DEBUG_OUT("underlying new client %p", event->client );
+							ASCanvas *cc = event->client->client_canvas ; 
+							if( cc && cc->root_x <= xbtn->x_root && cc->root_y <= xbtn->y_root &&
+								cc->root_x+cc->width > xbtn->x_root && cc->root_y+cc->height > xbtn->y_root )
+							{
+								event->w = xbtn->window = event->client->w ; 
+								xbtn->subwindow = None ; 
+						        XSendEvent (dpy, event->client->w, False, 
+											(xbtn->type == ButtonPress)?ButtonPressMask:ButtonReleaseMask, &(event->x));
+								xbtn->subwindow = event->client->w ; 
+								xbtn->window = Scr.Root ;
+								/* xbtn->window should still be Root, so we can proxy it down to the app */
+							}else
+							{
+								event->w = xbtn->window = event->client->frame ; 
+								xbtn->subwindow = None ;
+							}
+					    	LOCAL_DEBUG_OUT("new event window %lX", event->w );
+							event->context = C_NO_CONTEXT ;						
+						}
+						break;
+					}
+				}
+			}
+#endif
+		}else if( event->w == Scr.ServiceWin ||
 			event->w == Scr.SizeWindow )
 		{
 	    	event->context = C_ROOT ; 
@@ -348,10 +404,17 @@ DigestEvent( ASEvent *event )
 		{	
     		event->widget = Scr.RootCanvas ;
 			event->client = NULL ;
-		}else
+		}else if(event->client == NULL )
 		{
-			event->widget = NULL ;	
-			event->client = window2ASWindow( event->w );
+			if( (event->eclass & ASE_POINTER_EVENTS) != 0 && is_balloon_click( &(event->x) ) != NULL  )
+			{	
+				event->client = NULL ;
+				event->widget = NULL ;	 
+			}else
+			{	
+				event->widget = NULL ;	
+				event->client = window2ASWindow( event->w );
+			}
 		}	 
 	}
 
@@ -365,6 +428,8 @@ DigestEvent( ASEvent *event )
         ASTBarData *pointer_bar = NULL ;
 		int pointer_root_x = xk->x_root;
 		int pointer_root_y = xk->y_root;
+		static int last_pointer_root_x = -1, last_pointer_root_y = -1; 
+		int tbar_side = ASWIN_HFLAGS(asw, AS_VerticalTitle)?FR_W:FR_N ; 
 
         /* Since key presses and button presses are grabbed in the frame
          * when we have re-parented windows, we need to find out the real
@@ -406,7 +471,12 @@ DigestEvent( ASEvent *event )
 
 			}
 
-            if( w != asw->frame )
+	        if (ASWIN_GET_FLAGS(asw, AS_Shaded) && canvas != asw->frame_sides[tbar_side] )
+			{
+				event->context = C_NO_CONTEXT ;				
+				if( asw->frame_sides[tbar_side] )
+					XRaiseWindow( dpy, asw->frame_sides[tbar_side]->w );
+			}else if( w != asw->frame )
             {
                 if( event->w == asw->frame )
                 {
@@ -438,6 +508,11 @@ DigestEvent( ASEvent *event )
                         }
                 }
             }
+			if( event->context == C_NO_CONTEXT && get_flags(Scr.Feel.flags, ClickToFocus) )
+			{
+				w = asw->frame ; 
+				event->context = C_FRAME ;
+			}	 
             event->w = w ;
         }else
         {
@@ -463,8 +538,21 @@ DigestEvent( ASEvent *event )
             }
         }
 
-        on_astbar_pointer_action( pointer_bar, event->context, (event->x.type == LeaveNotify) );
-		apply_context_cursor( w, &(Scr.Feel), event->context );
+		if( pointer_bar != NULL ) 
+		{	
+        	on_astbar_pointer_action( pointer_bar, event->context, 
+								  	(event->x.type == LeaveNotify),
+								  	(last_pointer_root_x != pointer_root_x || last_pointer_root_y != pointer_root_y) );
+		}		
+		if( event->x.type == LeaveNotify ) 
+		{
+			withdraw_active_balloon_from( TitlebarBalloons );
+		}
+		last_pointer_root_x = pointer_root_x ;
+		last_pointer_root_y = pointer_root_y ;
+
+		if( asw != NULL && w != asw->w && w != asw->frame && event->context != C_NO_CONTEXT )
+			apply_context_cursor( w, &(Scr.Feel), event->context );
         event->widget  = canvas ;
         /* we have to do this at all times !!!! */
         /* if( event->x.type == ButtonRelease && Scr.Windows->pressed )
@@ -567,7 +655,13 @@ DispatchEvent ( ASEvent *event, Bool deffered )
             * indicates that the warp is done */
             if( get_flags( AfterStepState, ASS_WarpingMode ) )
                 EndWarping();
-            HandleButtonPress (event, deffered);
+			if( event->x.xbutton.button > Button3 )
+			{/* buttons 4 and 5 are for scrollwheel */
+				ASInternalWindow *internal = event->client?event->client->internal:NULL ;
+				if( internal && internal->on_scroll_event )
+					internal->on_scroll_event( internal, event );
+			}else
+		        HandleButtonPress (event, deffered);
             break;
         case ButtonRelease:
             /* if warping, a button press, non-warp keypress, or pointer motion
@@ -663,15 +757,20 @@ DispatchEvent ( ASEvent *event, Bool deffered )
 void
 HandleFocusIn ( ASEvent *event )
 {
-    while (ASCheckTypedEvent (FocusIn, &event->x));
-    DigestEvent( event );
+    int events_count = 0 ;
+	while (ASCheckTypedEvent (FocusIn, &event->x)) events_count++;
+	if( events_count > 0 ) 
+    	DigestEvent( event );
 
     if( get_flags( AfterStepState, ASS_WarpingMode ) )
         ChangeWarpingFocus( event->client );
 
+	LOCAL_DEBUG_OUT( "focused = %p, this event for %p", Scr.Windows->focused, event->client );
     if( Scr.Windows->focused != event->client )
+	{
+		LOCAL_DEBUG_OUT( "CHANGE Scr.Windows->focused from %p to NULL", Scr.Windows->focused );
         Scr.Windows->focused = NULL ;
-
+	}
     if( event->client == NULL && get_flags(AfterStepState, ASS_HousekeepingMode))
         return;
     /* note that hilite_aswindow changes value of Scr.Hilite!!! */
@@ -690,30 +789,39 @@ HandleKeyPress ( ASEvent *event )
 	FuncKey      *key;
     XKeyEvent *xk = &(event->x.xkey);
     unsigned int modifier = (xk->state & nonlock_mods);
+	int m ; 
 
 	/* Here's a real hack - some systems have two keys with the
 		* same keysym and different keycodes. This converts all
 		* the cases to one keycode. */
-    xk->keycode = XKeysymToKeycode (dpy, XKeycodeToKeysym (dpy, xk->keycode, 0));
+	for( m = 0 ; m < 8 ; ++m ) 
+	{	
+		KeySym keysym = XKeycodeToKeysym (dpy, xk->keycode, m); 
+		int keycode ; 
+		if( keysym == NoSymbol ) 
+			continue;
+    	if( (keycode = XKeysymToKeycode (dpy, keysym)) == 0 )
+			continue;
+		xk->keycode = keycode;
 
-    for (key = Scr.Feel.FuncKeyRoot; key != NULL; key = key->next)
-	{
-        if ((key->keycode == xk->keycode) &&
-			((key->mods == (modifier & (~LockMask))) ||
-             (key->mods == AnyModifier)) && (key->cont & event->context))
+    	for (key = Scr.Feel.FuncKeyRoot; key != NULL; key = key->next)
 		{
-			/* check if the warp key was pressed */
-            ExecuteFunction (key->fdata, event, -1);
-			return;
+        	if ((key->keycode == xk->keycode) &&
+				((key->mods == (modifier & (~LockMask))) ||
+             	(key->mods == AnyModifier)) && (key->cont & event->context))
+			{
+				/* check if the warp key was pressed */
+            	ExecuteFunction (key->fdata, event, -1);
+				return;
+			}
 		}
 	}
-
 	/* if a key has been pressed and it's not one of those that cause
 	   warping, we know the warping is finished */
     if( get_flags( AfterStepState, ASS_WarpingMode ) )
         EndWarping();
 
-	LOCAL_DEBUG_OUT( "client = %p", event->client );
+	LOCAL_DEBUG_OUT( "client = %p, context = %s", event->client, context2text((event)->context));
 	/* if we get here, no function key was bound to the key.  Send it
      * to the client if it was in a window we know about: */
     if (event->client)
@@ -759,6 +867,7 @@ HandlePropertyNotify (ASEvent *event)
     ASWindow       *asw;
     XPropertyEvent *xprop = &(event->x.xproperty);
     Atom atom = xprop->atom ;
+	XEvent prop_xev ; 
 
 	/* force updates for "transparent" windows */
     if (atom == _XROOTPMAP_ID && event->w == Scr.Root)
@@ -772,9 +881,11 @@ HandlePropertyNotify (ASEvent *event)
         }
         if(Scr.RootBackground && Scr.RootBackground->im != NULL )
         {
-            if( Scr.RootBackground->pmap && Scr.wmprops->root_pixmap == Scr.RootBackground->pmap );
+            if( Scr.RootBackground->pmap && Scr.wmprops->root_pixmap == Scr.RootBackground->pmap )
                 Scr.RootImage = dup_asimage(Scr.RootBackground->im) ;
         }
+		if( Scr.wmprops->as_root_pixmap != Scr.wmprops->root_pixmap ) 
+			set_as_background(Scr.wmprops, Scr.wmprops->root_pixmap );
 
         iterate_asbidirlist( Scr.Windows->clients, update_transp_iter_func, NULL, NULL, False );
 
@@ -790,10 +901,15 @@ HandlePropertyNotify (ASEvent *event)
 					move_menu (menu, (*menu).x, (*menu).y);
         }
 #endif
+		return;
     }
 
-    if( (asw = event->client) == NULL )
+    if( (asw = event->client) == NULL || ASWIN_GET_FLAGS( asw, AS_Dead ))
+	{
+		if( event->w != Scr.Root )
+			while (XCheckTypedWindowEvent (dpy, event->w, PropertyNotify, &prop_xev));
         return ;
+	}
     else
     {
 		char *prop_name = NULL;
@@ -804,28 +920,38 @@ HandlePropertyNotify (ASEvent *event)
     if( IsNameProp(atom))
     {
 		char *old_name = get_flags( asw->internal_flags, ASWF_NameChanged )?NULL:mystrdup( ASWIN_NAME(asw) );
+		
+		/* we want to check if there were some more events generated 
+		 * as window names tend to change multiple properties : */
+		while (XCheckTypedWindowEvent (dpy, asw->w, PropertyNotify, &prop_xev))
+			if( !IsNameProp(prop_xev.xproperty.atom) ) 
+			{	
+				XPutBackEvent( dpy, &prop_xev );
+				break;
+			}
+		
 		/*ASFlagType old_hflags = asw->hints->flags ; */
 		show_debug( __FILE__, __FUNCTION__, __LINE__, "name prop changed..." );
-        if( update_property_hints_manager( asw->w, xprop->atom,
-                                        Scr.Look.supported_hints,
-										Database,
-                                        asw->hints, asw->status ) )
+        if( get_flags( Scr.Feel.flags, FollowTitleChanges) )
+    	    on_window_hints_changed( asw );
+    	else if( update_property_hints_manager( asw->w, xprop->atom,
+             	                           		Scr.Look.supported_hints,
+												Database,
+                                        		asw->hints, asw->status ) )
         {
-            show_debug( __FILE__, __FUNCTION__, __LINE__, "New name is \"%s\", icon_name \"%s\", following title change ? %s", 
+            if( ASWIN_GET_FLAGS( asw, AS_Dead ) )
+				return;
+			show_debug( __FILE__, __FUNCTION__, __LINE__, "New name is \"%s\", icon_name \"%s\", following title change ? %s", 
 				        ASWIN_NAME(asw), ASWIN_ICON_NAME(asw), get_flags( Scr.Feel.flags, FollowTitleChanges)?"yes":"no" );
 	    	LOCAL_DEBUG_OUT( "hints flags = %lX, ShortLived ? %lX ", asw->hints->flags, ASWIN_HFLAGS( asw, AS_ShortLived ) );
 			if( old_name && strcmp( old_name, ASWIN_NAME(asw) ) != 0 )
 				set_flags( asw->internal_flags, ASWF_NameChanged );
             /* fix the name in the title bar */
-            if( get_flags( Scr.Feel.flags, FollowTitleChanges) )
-	    	    on_window_hints_changed( asw );
-	    	else
-	    	{
-				if (!ASWIN_GET_FLAGS(asw, AS_Iconic))
-           	    	on_window_title_changed( asw, True );
-	        	broadcast_window_name( asw );
-        		broadcast_icon_name( asw );
-		    }
+			if (!ASWIN_GET_FLAGS(asw, AS_Iconic))
+       	    	on_window_title_changed( asw, True );
+			broadcast_res_names( asw );
+        	broadcast_window_name( asw );
+       		broadcast_icon_name( asw );
         }
 		if( old_name )
 			free( old_name );
@@ -834,8 +960,13 @@ HandlePropertyNotify (ASEvent *event)
     }else if( atom == XA_WM_COMMAND || atom == XA_WM_CLIENT_MACHINE )
 	{
 		update_cmd_line_hints (asw->w, atom, asw->hints, asw->status );
+	}else if( atom == _XA_NET_WM_WINDOW_OPACITY )
+	{	
+	 	if( event->context == C_WINDOW && event->w != asw->frame ) 
+			on_window_opacity_changed( asw );
 	}else if( NeedToTrackPropChanges(atom) )
         on_window_hints_changed( asw );
+
     /* we have to do the complete refresh of hints, since we have to override WH_HINTS with database, etc. */
 }
 
@@ -859,7 +990,7 @@ HandleClientMessage (ASEvent *event)
         (event->x.xclient.data.l[0] == IconicState) &&
         !ASWIN_GET_FLAGS(event->client, AS_Iconic))
 	{
-        set_window_wm_state( event->client, True );
+        set_window_wm_state( event->client, True, False );
 #ifdef ENABLE_DND
 		/* Pass the event to the client window */
         if (event->x.xclient.window != event->client->w)
@@ -876,16 +1007,29 @@ HandleClientMessage (ASEvent *event)
 		ASFlagType extwm_flags = 0, as_flags = 0;
 		CARD32 props[2] ;
 		XClientMessageEvent *xcli = &(event->x.xclient) ;
+		
 		props[0] = xcli->data.l[1] ;
 		props[1] = xcli->data.l[2] ;
-
+		
 		translate_atom_list (&extwm_flags, EXTWM_State, &props[0], 2);
 		/* now we need to translate EXTWM flags into AS flags : */
 		as_flags = extwm_state2as_state_flags( extwm_flags );
 		if( xcli->data.l[0] == EXTWM_StateRemove ) 
+		{
 			as_flags = ASWIN_GET_FLAGS(event->client,as_flags);
-		else if( xcli->data.l[0] == EXTWM_StateAdd ) 
+		}else if( xcli->data.l[0] == EXTWM_StateAdd ) 
 			as_flags = as_flags&(~ASWIN_GET_FLAGS(event->client,as_flags));
+
+		if( props[0] == _XA_NET_WM_STATE_DEMANDS_ATTENTION ||
+			props[1] == _XA_NET_WM_STATE_DEMANDS_ATTENTION  ) 
+		{/* requires special treatment as it competes with ICCCM HintUrgency in WM_HINTS */
+			Bool set = True ; 
+			if( xcli->data.l[0] == EXTWM_StateRemove || 
+				(xcli->data.l[0] == EXTWM_StateToggle && ASWIN_GET_FLAGS(event->client,AS_Urgent)))  
+				set = False ; 
+			set_extwm_urgency_state( event->client->w, set );
+		}		
+		
 		if( as_flags != 0 ) 
 			toggle_aswindow_status(event->client, as_flags );
 	}	 
@@ -924,16 +1068,25 @@ HandleDestroyNotify (ASEvent *event )
  *  Procedure:
  *	HandleMapRequest - MapRequest event handler
  ************************************************************************/
+void
+delayed_add_window( void* vdata )
+{
+	if( window2ASWindow((Window)vdata) == NULL )
+		AddWindow ((Window)vdata, True);
+}
+
 void
 HandleMapRequest (ASEvent *event )
 {
     /* If the window has never been mapped before ... */
     if (event->client == NULL)
-    {
-        if( (event->client = AddWindow (event->w)) == NULL )
+    {/* lets delay handling map request in case client needs time to update its properties */
+		timer_new (200, delayed_add_window, (void*)event->w);	
+
+/*        if( (event->client = AddWindow (event->w, True)) == NULL ) */
             return;
     }else /* If no hints, or currently an icon, just "deiconify" */
-        set_window_wm_state( event->client, False );
+        set_window_wm_state( event->client, False, True );
 }
 
 /***********************************************************************
@@ -952,6 +1105,7 @@ HandleMapNotify ( ASEvent *event )
     if ( asw == NULL || event->w == Scr.Root )
         return;
 
+    LOCAL_DEBUG_OUT( "asw->w = %lX, event->w = %lX", asw->w, event->w );
     if( event->w != asw->w )
     {
         if( asw->wm_state_transition == ASWT_Withdrawn2Iconic && event->w == asw->status->icon_window )
@@ -962,11 +1116,18 @@ HandleMapNotify ( ASEvent *event )
     }
 
 	if( asw->wm_state_transition == ASWT_Withdrawn2Normal )
-		no_focus = (ASWIN_HFLAGS(asw, AS_FocusOnMap) == 0) ;
-    if( asw->wm_state_transition == ASWT_StableState )
+	{
+		if( ASWIN_HFLAGS(asw, AS_FocusOnMap) )
+			force_activation = True;
+		else
+			no_focus = True ;
+	}
+    LOCAL_DEBUG_OUT( "asw->wm_state_transition = %d", asw->wm_state_transition );
+
+	if( asw->wm_state_transition == ASWT_StableState )
     {
         if( ASWIN_GET_FLAGS( asw, AS_Iconic ) )
-            set_window_wm_state( asw, False );  /* client has requested deiconification */
+            set_window_wm_state( asw, False, False );  /* client has requested deiconification */
         return ;                                /* otherwise it is redundand event */
     }
     if( get_flags(asw->wm_state_transition, ASWT_FROM_ICONIC ) )
@@ -977,7 +1138,8 @@ HandleMapNotify ( ASEvent *event )
     ASWIN_CLEAR_FLAGS(asw, AS_IconMapped);
     ASWIN_CLEAR_FLAGS(asw, AS_Iconic);
     complete_wm_state_transition( asw, NormalState );
-	if( !no_focus )
+	LOCAL_DEBUG_OUT( "no_focus = %d, force_activation = %d, AcceptsFocus = %ld", no_focus, force_activation, ASWIN_HFLAGS(asw,AS_AcceptsFocus) );
+	if( !no_focus && ASWIN_FOCUSABLE(asw) )
     	activate_aswindow (asw, force_activation, False);
     broadcast_config( M_MAP, asw );
     /* finally reaches Normal state */
@@ -1031,12 +1193,12 @@ HandleUnmapNotify (ASEvent *event )
 
     grab_server();
     destroyed = ASCheckTypedWindowEvent ( event->w, DestroyNotify, &dummy) ;
-	LOCAL_DEBUG_OUT("wm_state_transition = 0x%X", event->client->wm_state_transition );
-	if( !get_flags( event->client->wm_state_transition, ASWT_FROM_WITHDRAWN ) )
-    	event->client->wm_state_transition = ASWIN_GET_FLAGS(event->client, AS_Iconic)?ASWT_Iconic2Withdrawn:ASWT_Normal2Withdrawn ;
+	LOCAL_DEBUG_OUT("wm_state_transition = 0x%X", asw->wm_state_transition );
+	if( !get_flags( asw->wm_state_transition, ASWT_FROM_WITHDRAWN ) )
+    	asw->wm_state_transition = ASWIN_GET_FLAGS(asw, AS_Iconic)?ASWT_Iconic2Withdrawn:ASWT_Normal2Withdrawn ;
 	else
-		event->client->wm_state_transition = ASWT_Withdrawn2Withdrawn ;
-    Destroy (event->client, destroyed);               /* do not need to mash event before */
+		asw->wm_state_transition = ASWT_Withdrawn2Withdrawn ;
+    Destroy (asw, destroyed);               /* do not need to mash event before */
     ungrab_server();
     ASFlush ();
 }
@@ -1054,45 +1216,53 @@ HandleButtonPress ( ASEvent *event, Bool deffered )
     Bool          AShandled = False;
     ASWindow     *asw = event->client ;
     XButtonEvent *xbtn = &(event->x.xbutton);
+	Bool 		  raise_on_click = False ; 
+    Bool 		  focus_accepted = False ;
+    Bool 		  eat_click = False;
+	Bool 		  activate_window = False ;
 
 	/* click to focus stuff goes here */
     if( asw != NULL )
     {
-        Bool eat_click = False;
+		LOCAL_DEBUG_OUT( "deferred = %d, button = %X", deffered, (event->context&(~C_TButtonAll)) );
         /* if all we do is pressing titlebar buttons - then we should not raise/focus window !!! */
-        if( !deffered && (event->context&(~C_TButtonAll)) != 0 )
+        if( !deffered )
         {
-            Bool focus_accepted = False ;
-            if (get_flags( Scr.Feel.flags, ClickToFocus) )
-      		{
-          		if ( asw != Scr.Windows->ungrabbed && (xbtn->state & nonlock_mods) == 0)
-                {
-                    if( Scr.Windows->focused != asw )
-                        focus_accepted = activate_aswindow( asw, False, False);
-                    if( focus_accepted && get_flags( Scr.Feel.flags, EatFocusClick ) )
-                        eat_click = True ;
-                }
-	        }
+			if( (event->context&(~C_TButtonAll)) != 0 )
+			{	
+            	if (get_flags( Scr.Feel.flags, ClickToFocus) )
+      			{
+					LOCAL_DEBUG_OUT( "asw = %p, ungrabbed = %p, nonlock_mods = %x", asw, Scr.Windows->ungrabbed, (xbtn->state & nonlock_mods) );
+          			if ( asw != Scr.Windows->ungrabbed && (xbtn->state & nonlock_mods) == 0)
+                	{
+						if( get_flags( Scr.Feel.flags, EatFocusClick ) )
+						{
+							if( Scr.Windows->focused != asw )
+								if( (focus_accepted = activate_aswindow( asw, False, False)) )
+                        			eat_click = True ;
+						}else if( Scr.Windows->focused != asw )
+							activate_window = True;
+						LOCAL_DEBUG_OUT( "eat_click = %d", eat_click );
+                	}
+	        	}
 
-            if ( get_flags(Scr.Feel.flags, ClickToRaise) )
-                if( Scr.Feel.RaiseButtons == 0 || (Scr.Feel.RaiseButtons & (1 << xbtn->button)))
-                    restack_window((asw),None,focus_accepted?Above:TopIf);
-
-            if (!ASWIN_GET_FLAGS(asw, AS_Iconic))
-  		    {
-      		    XSync (dpy, 0);
-          		XAllowEvents (dpy, (event->context == C_WINDOW) ? ReplayPointer : AsyncPointer, CurrentTime);
-	            XSync (dpy, 0);
-  		    }
+            	if ( get_flags(Scr.Feel.flags, ClickToRaise) )
+					raise_on_click = ( Scr.Feel.RaiseButtons == 0 || (Scr.Feel.RaiseButtons & (1 << xbtn->button))) ; 
+			}
+			
+			if (!ASWIN_GET_FLAGS(asw, AS_Iconic))
+  			{
+      			XSync (dpy, 0);
+        		XAllowEvents (dpy, (event->context == C_WINDOW) ? ReplayPointer : AsyncPointer, CurrentTime);
+	    		XSync (dpy, 0);
+  			}
 		} /* !deffered */
 
         press_aswindow( asw, event->context );
-
-        if( eat_click )
-            return;
     }
 
-	if( !deffered )
+
+	if( !deffered && !eat_click)
 	{
 		LOCAL_DEBUG_OUT( "checking for associated functions...%s","" );		
 	    /* we have to execute a function or pop up a menu : */
@@ -1111,6 +1281,7 @@ HandleButtonPress ( ASEvent *event, Bool deffered )
 	            if (MouseEntry->fdata != NULL)
 				{
       		        ExecuteFunction (MouseEntry->fdata, event, -1);
+					raise_on_click = False ;
 					AShandled = True;
 					break;
 				}
@@ -1118,8 +1289,16 @@ HandleButtonPress ( ASEvent *event, Bool deffered )
 			MouseEntry = MouseEntry->NextButton;
 		}
 	}
+
+	LOCAL_DEBUG_OUT( "ashandled = %d, context = %X", AShandled, (event->context&(C_CLIENT|C_TITLE)) );
+	if( activate_window && (!AShandled || (event->context&(C_CLIENT|C_TITLE)))) 
+		focus_accepted = activate_aswindow( asw, False, False);
+
+	if( raise_on_click && (asw->internal == NULL || (event->context&C_CLIENT) == 0 ) )
+    	restack_window(asw,None,focus_accepted?Above:TopIf);
+
 	/* GNOME this click hasn't been taken by AfterStep */
-    if (!deffered && !AShandled && xbtn->window == Scr.Root)
+    if (!deffered && !AShandled && !eat_click && xbtn->window == Scr.Root)
 	{
         XUngrabPointer (dpy, CurrentTime);
         XSendEvent (dpy, Scr.wmprops->wm_event_proxy, False, SubstructureNotifyMask, &(event->x));
@@ -1156,7 +1335,7 @@ HandleEnterNotify (ASEvent *event)
     if (ewp->window != Scr.Root)
     	if (ASCheckTypedWindowEvent ( ewp->window, LeaveNotify, &d))
 		{
-        	on_astbar_pointer_action( NULL, 0, True );
+        	on_astbar_pointer_action( NULL, 0, True, False );
         	if ((d.xcrossing.mode == NotifyNormal) && (d.xcrossing.detail != NotifyInferior))
 				return;
 		}
@@ -1190,9 +1369,9 @@ HandleEnterNotify (ASEvent *event)
     if (asw == NULL )
 		return;
 
-    if (ASWIN_HFLAGS(asw,AS_AcceptsFocus))
+    if (ASWIN_FOCUSABLE(asw))
 	{
-        if (!get_flags(Scr.Feel.flags, ClickToFocus))
+        if (!get_flags(Scr.Feel.flags, ClickToFocus) || asw->internal != NULL )
 		{
             if (Scr.Windows->focused != asw)
                 activate_aswindow(asw, False, False);
@@ -1244,6 +1423,7 @@ HandleConfigureRequest ( ASEvent *event )
     XConfigureRequestEvent *cre = &(event->x.xconfigurerequest);
     ASWindow *asw = event->client ;
     XWindowChanges xwc;
+    unsigned long xwcm;
 
     /*
 	 * According to the July 27, 1988 ICCCM draft, we should ignore size and
@@ -1252,17 +1432,57 @@ HandleConfigureRequest ( ASEvent *event )
 	 * to configuration requests for windows which have never been mapped.
 	 */
 
+	LOCAL_DEBUG_OUT( "cre={0x%lx, geom = %dx%d%+d%+d} ", cre->value_mask, cre->width, cre->height,
+	cre->x, cre->y );
     if (asw == NULL)
 	{
-        unsigned long xwcm;
         xwcm = cre->value_mask & (CWX | CWY | CWWidth | CWHeight | CWBorderWidth);
-		xwc.x = cre->x;
-		xwc.y = cre->y;
-		xwc.width = cre->width;
-		xwc.height = cre->height;
-		xwc.border_width = cre->border_width;
-		LOCAL_DEBUG_OUT( "Configuring untracked window %lX to %dx%d%+d%+d and bw = %d, (flags=%lX)", (unsigned long)event->w, cre->width, cre->height, cre->x, cre->y, cre->border_width, xwcm );
-        XConfigureWindow (dpy, event->w, xwcm, &xwc);
+		if( xwcm == 0 ) 
+		{
+			LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for untracked window %lX - no supported changes detected.", (unsigned long)event->w );
+		}else
+		{
+			xwc.x = cre->x;
+			xwc.y = cre->y;
+			xwc.width = cre->width;
+			xwc.height = cre->height;
+			xwc.border_width = cre->border_width;
+			LOCAL_DEBUG_OUT( "Configuring untracked window %lX to %dx%d%+d%+d and bw = %d, (flags=%lX)", (unsigned long)event->w, cre->width, cre->height, cre->x, cre->y, cre->border_width, xwcm );
+        	XConfigureWindow (dpy, event->w, xwcm, &xwc);
+			ASSync( False);
+		}
+		return;
+	}
+
+	if( asw->icon_canvas && event->w == asw->icon_canvas->w ) 
+	{ /* we really should ignore that ! - let's see how it will play out */
+	  /* we may need to add code to iconbox to handle custom icon geometry for client icons */
+        xwcm = cre->value_mask & (CWWidth | CWHeight);
+		if( xwcm == 0 ) 
+		{
+			LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for iconic window %lX - no supported changes detected.", (unsigned long)event->w );
+		}else
+		{
+			xwc.width = cre->width;
+			xwc.height = cre->height;
+			LOCAL_DEBUG_OUT( "Configuring iconic window %lX to %dx%d, (flags=%lX)", (unsigned long)event->w, cre->width, cre->height, xwcm );
+        	XConfigureWindow (dpy, event->w, xwcm, &xwc);
+			ASSync( False);
+		}
+		return;
+	}
+	
+	if( event->w != asw->w ) 
+	{
+		LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for window %lX. Not a client or client's icon!", (unsigned long)event->w );
+		return;
+	}
+	
+	if( ( ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) && !get_flags(cre->value_mask, CWWidth|CWHeight))||
+		ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )
+	{	
+		LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for client %p as required by hints", asw );
+		SendConfigureNotify(asw);
 		return;
 	}
 
@@ -1280,25 +1500,28 @@ HandleConfigureRequest ( ASEvent *event )
     if( cre->value_mask & (CWWidth|CWHeight|CWX|CWY) )
     {
         XRectangle new_anchor = asw->anchor;
-        int grav_x, grav_y ;
-        get_gravity_offsets (asw->hints, &grav_x, &grav_y);
-
 
         if( cre->value_mask&CWWidth )
             new_anchor.width = cre->width ;
         if( cre->value_mask&CWHeight )
             new_anchor.height = cre->height ;
-        if( cre->value_mask&CWX )
-            new_anchor.x = make_anchor_pos ( asw->status, cre->x, new_anchor.width, Scr.Vx, grav_x, Scr.VxMax+Scr.MyDisplayWidth );
-        if( cre->value_mask&CWY )
-            new_anchor.y = make_anchor_pos ( asw->status, cre->y, new_anchor.height, Scr.Vy, grav_y, Scr.VyMax+Scr.MyDisplayHeight );
-
+		if( !ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) )
+		{
+    	    int grav_x, grav_y ;
+	        get_gravity_offsets (asw->hints, &grav_x, &grav_y);
+        	if( cre->value_mask&CWX )
+            	new_anchor.x = make_anchor_pos ( asw->status, cre->x, new_anchor.width, Scr.Vx, grav_x, Scr.VxMax+Scr.MyDisplayWidth );
+        	if( cre->value_mask&CWY )
+            	new_anchor.y = make_anchor_pos ( asw->status, cre->y, new_anchor.height, Scr.Vy, grav_y, Scr.VyMax+Scr.MyDisplayHeight );
+		}
 LOCAL_DEBUG_OUT( "old anchor(%dx%d%+d%+d), new_anchor(%dx%d%+d%+d)", asw->anchor.width, asw->anchor.height, asw->anchor.x, asw->anchor.y, new_anchor.width, new_anchor.height, new_anchor.x, new_anchor.y );
         validate_window_anchor( asw, &new_anchor );
 LOCAL_DEBUG_OUT( "validated_anchor(%dx%d%+d%+d)", new_anchor.width, new_anchor.height, new_anchor.x, new_anchor.y );
         asw->anchor = new_anchor ;
         on_window_anchor_changed( asw );
 		enforce_avoid_cover( asw );
+        if( (cre->value_mask&(CWWidth|CWHeight)) == 0 )
+			SendConfigureNotify(asw);		
     }
 }
 
@@ -1370,7 +1593,7 @@ void HandleShmCompletion(ASEvent *event)
 {
 #ifdef XSHMIMAGE
     XShmCompletionEvent  *sev = (XShmCompletionEvent*) &(event->x);
-	LOCAL_DEBUG_OUT( "XSHMIMAGE> EVENT : offset   %ld(%lx)", (long)sev->offset, (unsigned long)(sev->offset) );
+	LOCAL_DEBUG_OUT( "XSHMIMAGE> EVENT : offset   %ld(%lx), shmseg = %lx", (long)sev->offset, (unsigned long)(sev->offset), sev->shmseg );
 	if( !is_background_xfer_ximage( sev->shmseg ) )
 		destroy_xshm_segment( sev->shmseg );
 #endif /* SHAPE */
@@ -1384,7 +1607,7 @@ void HandleShmCompletion(ASEvent *event)
  *
  ****************************************************************************/
 void
-afterstep_wait_pipes_input()
+afterstep_wait_pipes_input(int timeout_sec)
 {
 	fd_set        in_fdset, out_fdset;
 	int           retval;
@@ -1427,6 +1650,12 @@ afterstep_wait_pipes_input()
 	/* watch for timeouts */
 	if (timer_delay_till_next_alarm ((time_t *) & tv.tv_sec, (time_t *) & tv.tv_usec))
 		t = &tv;
+	else if( timeout_sec > 0 )
+	{
+		t = &tv ; 
+		tv.tv_sec = timeout_sec ; 
+		tv.tv_usec = 0 ;
+	}
     retval = PORTABLE_SELECT(min (max_fd + 1, fd_width),&in_fdset,&out_fdset,NULL,t);
 
 	if (retval > 0)
