@@ -87,6 +87,8 @@ free_asmenu_item( ASMenuItem *item )
 {
     if( item->bar )
         destroy_astbar(&(item->bar) );
+	if( item->icon )
+	    safe_asimage_destroy( item->icon );
 	free_func_data (&(item->fdata));
 }
 
@@ -123,6 +125,8 @@ destroy_asmenu(ASMenu **pmenu)
             Window w = menu->main_canvas->w ;
 LOCAL_DEBUG_CALLER_OUT( "top(%p)->supermenu(%p)->menu(%p)->submenu(%p)", ASTopmostMenu, menu->supermenu, menu, menu->submenu );
 
+	        while (timer_remove_by_data (menu));
+
             if( menu->supermenu && menu->supermenu->submenu == menu )
                 menu->supermenu->submenu = NULL ;
             else if( ASTopmostMenu == menu )
@@ -158,6 +162,14 @@ LOCAL_DEBUG_CALLER_OUT( "top(%p)->supermenu(%p)->menu(%p)->submenu(%p)", ASTopmo
                 free( menu->name );
             if( menu->title )
                 free( menu->title );
+
+			destroy_asballoon( &(menu->comment_balloon) );
+			destroy_asballoon( &(menu->item_balloon) );
+			
+			if( menu->volitile_menu_data != NULL ) 
+				destroy_menu_data( &(menu->volitile_menu_data) );
+
+			
             menu->magic = 0 ;
             free( menu );
             *pmenu = NULL ;
@@ -203,8 +215,9 @@ set_asmenu_item_data( ASMenuItem *item, MenuDataItem *mdi )
     item->source = mdi ;
 
     if( item->bar == NULL )
+	{
         item->bar = create_astbar();
-	else
+	}else
 		delete_astbar_tile( item->bar, -1 );  /* delete all tiles */
 
 
@@ -249,16 +262,17 @@ LOCAL_DEBUG_OUT( "item(\"%s\")->minipixmap(\"%s\")->icon(%p)", mdi->item?mdi->it
 
     /* optional menu items : */
     /* add label */
-    if( mdi->item )
-	{	
-        add_astbar_label( item->bar, 3, 0, 0, ALIGN_LEFT|ALIGN_VCENTER, 0, 0, mdi->item,
-		mdi->fdata->name_encoding );
-		item->first_sym = mdi->item[0] ;
+	{
+		int encoding = get_flags( mdi->flags, MD_NameIsUTF8)? AS_Text_UTF8: mdi->fdata->name_encoding ;
+    	if( mdi->item )
+		{	
+        	add_astbar_label( item->bar, 3, 0, 0, ALIGN_LEFT|ALIGN_VCENTER, 0, 0, mdi->item, encoding );
+			item->first_sym = mdi->item[0] ;
+		}
+    	/* add hotkey */
+    	if( mdi->item2 )
+        	add_astbar_label( item->bar, 4, 0, 0, ALIGN_RIGHT|ALIGN_VCENTER, 0, 0, mdi->item2, encoding );
 	}
-    /* add hotkey */
-    if( mdi->item2 )
-        add_astbar_label( item->bar, 4, 0, 0, ALIGN_RIGHT|ALIGN_VCENTER, 0, 0, mdi->item2, mdi->fdata->name_encoding );
-
     item->flags = 0 ;
 
 	if( IsDynamicPopup(mdi->fdata->func) )
@@ -299,6 +313,12 @@ LOCAL_DEBUG_OUT( "item.bar(%p)->look(%p)", item->bar, look );
     if( item->bar == NULL )
         return False;
 
+	if( get_flags( look->flags, TxtrMenuItmInd ) )
+		clear_flags( item->bar->state, BAR_FLAGS_CROP_BACK );
+	else
+		set_flags( item->bar->state, BAR_FLAGS_CROP_UNFOCUSED_BACK );
+
+
     item->bar->h_spacing = DEFAULT_MENU_SPACING ;
     item->bar->h_border = DEFAULT_MENU_ITEM_HBORDER ;
     item->bar->v_border = DEFAULT_MENU_ITEM_VBORDER ;
@@ -398,6 +418,7 @@ render_asmenu_bars( ASMenu *menu, Bool force )
     Bool rendered = False;
     if( menu->main_canvas->height > 1 && menu->main_canvas->width > 1 )
     {
+		ASImage *cache = NULL ; 
         START_LONG_DRAW_OPERATION;
         for( i = 0 ; i < menu->items_num ; ++i )
         {
@@ -416,7 +437,7 @@ render_asmenu_bars( ASMenu *menu, Bool force )
                 bar->win_y + (int)(bar->height) > 0 &&
                 (int)(bar->win_y) > prev_y )
             {
-                if( render_astbar( bar, menu->main_canvas ) )
+                if( render_astbar_cached_back( bar, menu->main_canvas, &cache, NULL ) )
                     rendered = True ;
             }
             prev_y = bar->win_y ;
@@ -424,10 +445,10 @@ render_asmenu_bars( ASMenu *menu, Bool force )
 		if( menu->visible_items_num < menu->items_num )
 		{
 			if( force || DoesBarNeedsRendering(menu->scroll_up_bar))
-            	if( render_astbar( menu->scroll_up_bar, menu->main_canvas ) )
+            	if( render_astbar_cached_back( menu->scroll_up_bar, menu->main_canvas, &cache, NULL ) )
                 	rendered = True ;
 			if( force || DoesBarNeedsRendering(menu->scroll_down_bar))
-            	if( render_astbar( menu->scroll_down_bar, menu->main_canvas ) )
+            	if( render_astbar_cached_back( menu->scroll_down_bar, menu->main_canvas, &cache, NULL ) )
                 	rendered = True ;
 		}
         STOP_LONG_DRAW_OPERATION;
@@ -436,8 +457,10 @@ render_asmenu_bars( ASMenu *menu, Bool force )
         {
             update_canvas_display( menu->main_canvas );
             ASSync(False);
-			menu->rendered = True ;
+			set_flags(	menu->state, AS_MenuRendered);
         }
+		if( cache ) 
+			safe_asimage_destroy( cache );
     }
 }
 
@@ -475,13 +498,14 @@ extract_recent_subitems( char *submenu_name, MenuDataItem **subitems, unsigned i
 }
 
 void
-set_asmenu_data( ASMenu *menu, MenuData *md, Bool first_time )
+set_asmenu_data( ASMenu *menu, MenuData *md, Bool first_time, Bool show_unavailable, int recent_items )
 {
     int items_num = md->items_num ;
     int i = 0 ;
     int real_items_num = 0;
     int max_icon_size  = 0;
     MenuDataItem **subitems = NULL ;
+	MenuDataItem *title_mdi = NULL; 
 
 
     if( menu->items_num < items_num )
@@ -495,27 +519,52 @@ set_asmenu_data( ASMenu *menu, MenuData *md, Bool first_time )
         free( menu->title );
         menu->title = NULL;
     }
+	clear_flags( menu->state, AS_MenuTitleIsUTF8|AS_MenuNameIsUTF8 );
+	
+	if( get_flags( md->flags, MD_NameIsUTF8 ) )
+		set_flags( menu->state, AS_MenuNameIsUTF8 );
 
     if( items_num > 0 )
     {
         MenuDataItem *mdi = md->first ;
+		if( md->recent_items >= 0 )
+			recent_items = md->recent_items ;
+        if( recent_items > 0 )
+            subitems = safecalloc( recent_items, sizeof(MenuDataItem*));
 
-        if( md->recent_items > 0 )
-            subitems = safecalloc( md->recent_items, sizeof(MenuDataItem*));
+		LOCAL_DEBUG_OUT( "show_unavailable = %d", show_unavailable );
 
         for(mdi = md->first; real_items_num < items_num && mdi != NULL ; mdi = mdi->next )
             if( mdi->fdata->func == F_TITLE && menu->title == NULL )
             {
+				title_mdi = mdi ; 
                 menu->title = mystrdup( mdi->item );
+				if( get_flags( mdi->flags, MD_NameIsUTF8 ) )
+					set_flags( menu->state, AS_MenuTitleIsUTF8 );
             }else
             {
                 ASMenuItem *item = &(menu->items[real_items_num]);
-                set_asmenu_item_data( item, mdi );
+				
+				if( !show_unavailable ) 
+				{
+					if( get_flags( mdi->flags, MD_Disabled ) )
+						continue;
+					if( mdi->fdata->func == F_NOP  )
+					{	
+						int len = mdi->item?strlen(mdi->item):0 ;
+						if(  mdi->item2 )
+							len += strlen(mdi->item2);
+						if( len > 0 )
+							continue;
+					}
+				}
 
+                set_asmenu_item_data( item, mdi );
                 ++real_items_num;
+
                 if( item->fdata.func == F_POPUP && !get_flags( item->flags, AS_MenuItemDisabled ) && subitems != NULL )
                 {
-                    int used = extract_recent_subitems( FDataPopupName(item->fdata), subitems, md->recent_items );
+                    int used = extract_recent_subitems( FDataPopupName(item->fdata), subitems, recent_items );
                     if( used > 0 )
                     {
                         items_num += used ;
@@ -568,6 +617,27 @@ set_asmenu_data( ASMenu *menu, MenuData *md, Bool first_time )
 	}else
 		menu->selected_item = -1 ;
     menu->pressed_item = -1;
+	
+	{
+		char *comment = md->comment ; 
+		int encoding = get_flags( md->flags, MD_CommentIsUTF8)? AS_Text_UTF8 : AS_Text_ASCII ;
+		if( title_mdi != NULL && title_mdi->comment != NULL ) 
+		{
+			encoding = get_flags( title_mdi->flags, MD_CommentIsUTF8)? AS_Text_UTF8 : AS_Text_ASCII ;
+			comment = title_mdi->comment ; 
+		}
+		if( comment ) 
+		{
+			if( menu->comment_balloon == NULL )
+				menu->comment_balloon = create_asballoon_with_text_for_state ( MenuBalloons, NULL, comment, encoding);
+			else
+			{
+				balloon_set_text (menu->comment_balloon, comment, encoding);
+				clear_flags( menu->state, AS_MenuBalloonShown);
+			}
+		}else
+			destroy_asballoon( &(menu->comment_balloon) );
+	}
 }
 
 void
@@ -712,11 +782,14 @@ set_menu_item_used( ASMenu *menu, MenuDataItem *mdi )
 
 
 void set_asmenu_scroll_position( ASMenu *menu, int pos );
+static void menu_item_balloon_timer_handler (void *data);
 
 void
-select_menu_item( ASMenu *menu, int selection )
+select_menu_item( ASMenu *menu, int selection, Bool render )
 {
+	Bool needs_scrolling ;
 LOCAL_DEBUG_CALLER_OUT( "%p,%d", menu, selection );
+	
     if( AS_ASSERT(menu) || menu->items_num == 0 )
         return;
     if( selection < 0 )
@@ -724,22 +797,65 @@ LOCAL_DEBUG_CALLER_OUT( "%p,%d", menu, selection );
     else if( selection >= (int)menu->items_num )
         selection = menu->items_num - 1 ;
 
+	needs_scrolling = ( selection < menu->top_item || selection >= menu->top_item + menu->visible_items_num ) ;
     if( selection != menu->selected_item )
     {
+		if( menu->item_balloon )
+		{
+	        while (timer_remove_by_data (menu));
+			withdraw_balloon( menu->item_balloon );
+		}
         close_asmenu_submenu( menu );
 		if( menu->selected_item >= 0 )
-        	set_astbar_focused( menu->items[menu->selected_item].bar, menu->main_canvas, False );
+        	set_astbar_focused( menu->items[menu->selected_item].bar, NULL/*needs_scrolling?NULL:menu->main_canvas*/, False );
     }
-    set_astbar_focused( menu->items[selection].bar, menu->main_canvas, True );
+    set_astbar_focused( menu->items[selection].bar, NULL/*needs_scrolling?NULL:menu->main_canvas*/, True );
     menu->selected_item = selection ;
 
     if( selection < menu->top_item )
-        set_asmenu_scroll_position( menu, MAX(selection-1, 0) );
+        set_asmenu_scroll_position( menu, MAX(selection, 0) );
     else if( selection >= menu->top_item + menu->visible_items_num )
         set_asmenu_scroll_position( menu, (selection-menu->visible_items_num)+1);
-    render_asmenu_bars(menu, False);
+	else if( render )
+	    render_asmenu_bars(menu, False);
+
+	if( menu->items[selection].source )
+	{
+		LOCAL_DEBUG_OUT( "selection func = %ld", menu->items[selection].fdata.func );
+		if( menu->items[selection].source->comment ) 
+		{
+			int	encoding = get_flags( menu->items[selection].source->flags, MD_CommentIsUTF8)? AS_Text_UTF8 : AS_Text_ASCII ;
+			if( menu->item_balloon == NULL )
+				menu->item_balloon = create_asballoon_for_state ( MenuBalloons, NULL);
+
+	        while (timer_remove_by_data (menu));
+			balloon_set_text (menu->item_balloon, menu->items[selection].source->comment, encoding);
+        	timer_new (MenuBalloons->look.Delay, &menu_item_balloon_timer_handler, (void *)menu);
+		}else if( menu->items[selection].fdata.func == F_CHANGE_BACKGROUND_FOREIGN || 
+				  menu->items[selection].fdata.func == F_CHANGE_BACKGROUND ) 
+		{
+			int delay = MenuBalloons->look.Delay ; 
+			LOCAL_DEBUG_OUT( "change background func = \"%s\"", menu->items[selection].fdata.text );
+
+			if( menu->item_balloon == NULL )
+				menu->item_balloon = create_asballoon_for_state ( MenuBalloons, NULL);
+	        while (timer_remove_by_data (menu));
+			balloon_set_image_from_file (menu->item_balloon, menu->items[selection].fdata.text );
+        	timer_new (delay>1000?delay-1000:delay, &menu_item_balloon_timer_handler, (void *)menu);
+		}
+	}
 }
 
+static void
+menu_item_balloon_timer_handler (void *data)
+{
+    ASMenu      *menu = (ASMenu *) data;
+
+	if( menu && menu->item_balloon ) 
+		display_balloon_nodelay( menu->item_balloon );	
+}
+
+
 void
 set_asmenu_scroll_position( ASMenu *menu, int pos )
 {
@@ -788,11 +904,11 @@ LOCAL_DEBUG_OUT("adj_pos(%d)->curr_y(%d)->items_num(%d)->vis_items_num(%d)->sel_
 	menu->top_item = first_item ;
 
 	if( menu->selected_item < menu->top_item )
-        select_menu_item( menu, menu->top_item );
+        select_menu_item( menu, menu->top_item, False );
     else if( menu->selected_item >= 0 )
 	{
 		if( menu->visible_items_num > 0 && menu->selected_item >= (int)menu->top_item + (int)menu->visible_items_num )
-        select_menu_item( menu, menu->top_item + menu->visible_items_num - 1 );
+        select_menu_item( menu, menu->top_item + menu->visible_items_num - 1, False );
 	}
     render_asmenu_bars(menu, False);
 }
@@ -842,7 +958,12 @@ LOCAL_DEBUG_CALLER_OUT( "%p, %d", menu, item_no );
 	*/      close_asmenu_submenu( menu );
         	menu->submenu = run_submenu( menu, submenu, x, y);
 			if( item->fdata.func != F_POPUP )
-    			destroy_menu_data( &submenu );
+			{
+				if( menu->submenu == NULL ) 
+	    			destroy_menu_data( &submenu );
+				else
+					menu->submenu->volitile_menu_data = submenu ;
+			}
 		}
 	}
 }
@@ -870,12 +991,12 @@ LOCAL_DEBUG_CALLER_OUT( "%p,%d", menu, pressed );
 			if( keyboard_event ) 
 			{   /* don't press if keyboard selection */
 				if( pressed != menu->selected_item )
-					select_menu_item( menu, pressed );
+					select_menu_item( menu, pressed, False );
 
 			}else if( pressed != menu->selected_item )
             {
                 set_astbar_pressed( menu->items[pressed].bar, NULL, True );/* don't redraw yet */
-                select_menu_item( menu, pressed );  /* this one updates display already */
+                select_menu_item( menu, pressed, False );  /* this one updates display already */
             }else
                 set_astbar_pressed( menu->items[pressed].bar, menu->main_canvas, True );
             set_menu_item_used( menu, menu->items[pressed].source );
@@ -894,8 +1015,8 @@ LOCAL_DEBUG_CALLER_OUT( "%p,%d", menu, pressed );
         }
     }
     render_asmenu_bars(menu, False);
-LOCAL_DEBUG_OUT( "pressed(%d)->old_pressed(%d)->focused(%d)", pressed, menu->pressed_item, menu->focused );
-    if( menu->focused )
+LOCAL_DEBUG_OUT( "pressed(%d)->old_pressed(%d)->focused(%d)", pressed, menu->pressed_item, get_flags(menu->state, AS_MenuFocused)?1:0 );
+    if( get_flags(menu->state, AS_MenuFocused) )
 	{	
 		int item_no = keyboard_event?pressed:menu->pressed_item ;
     	
@@ -907,7 +1028,7 @@ LOCAL_DEBUG_OUT( "pressed(%d)->old_pressed(%d)->focused(%d)", pressed, menu->pre
         	{
             	set_menu_item_used( menu, item->source );
             	ExecuteFunctionForClient( &(item->fdata), menu->client_window );
-            	if( !menu->pinned )
+            	if( !get_flags(menu->state, AS_MenuPinned) )
                 	close_asmenu( &ASTopmostMenu );
         	}
     	}
@@ -979,7 +1100,10 @@ LOCAL_DEBUG_CALLER_OUT( "%p, %p, %d", asiw, data, focused );
     {
         /* TODO : hilite/unhilite selected item, and
          * withdraw non-pinned menu if it has no submenus */
-        menu->focused = focused ;
+		if( focused ) 
+		 	set_flags(menu->state, AS_MenuFocused);
+		else
+			clear_flags(menu->state, AS_MenuFocused);
     }
 }
 
@@ -1028,6 +1152,70 @@ LOCAL_DEBUG_OUT( "pointer(%d,%d)", px, py );
     }
 }
 
+static void 
+handle_menu_selection( ASMenu *menu, int button_state, int px, int selection, Bool render)
+{
+    if( selection != menu->selected_item )
+    {
+        if( button_state & (Button1Mask|Button2Mask|Button3Mask) )
+            press_menu_item( menu, selection, False );
+        else
+            select_menu_item( menu, selection, render );
+    }
+    if( px > menu->main_canvas->width - ( menu->arrow_space + DEFAULT_MENU_SPACING ) &&
+        menu->submenu == NULL )
+        run_item_submenu( menu, selection );
+
+}
+
+void
+on_menu_scroll_event( ASInternalWindow *asiw, ASEvent *event )
+{
+    ASMenu   *menu = (ASMenu*)(asiw->data) ;
+    if( menu != NULL && menu->magic == MAGIC_ASMENU && event)
+    {
+		if(  menu->items_num >= menu->visible_items_num )
+		{
+			XButtonEvent *xbtn = &(event->x.xbutton);
+			int selection = menu->selected_item ; 
+			int top = menu->top_item ;
+			int px_offset = menu->main_canvas->root_x+(int)menu->main_canvas->bw ;
+			int px, py ;
+			Bool needs_scrolling ;
+			
+			ASQueryPointerRootXY( &px, &py );
+			if( xbtn->button == Button4 )		
+			{
+				if( selection > 0 ) 
+				{
+					--selection ;
+					py -= menu->item_height ;
+				}
+				if( selection < menu->top_item )
+					--top ;
+			}else
+			{
+				if( selection+1 < menu->items_num ) 
+				{
+					++selection ;
+					py += menu->item_height ;
+				}
+				if( selection > menu->top_item + menu->visible_items_num - 1 )
+					++top ;
+			}
+			needs_scrolling = (top != menu->top_item);
+			if( selection != menu->selected_item )
+			{
+				handle_menu_selection( menu, xbtn->state, px-px_offset, selection, !needs_scrolling);
+				if( !needs_scrolling ) 
+					XWarpPointer (dpy, None, Scr.Root, 0, 0, 0, 0, px, py);
+			}
+//			if( needs_scrolling ) 
+//				set_asmenu_scroll_position( menu, top );
+		}
+	}
+}
+
 /* Motion notify : */
 void
 on_menu_pointer_event( ASInternalWindow *asiw, ASEvent *event )
@@ -1043,9 +1231,16 @@ on_menu_pointer_event( ASInternalWindow *asiw, ASEvent *event )
 		XEvent tmp_e ;
 		if( ASCheckTypedWindowEvent(canvas->w,LeaveNotify,&tmp_e) )
 		{
+        	if( menu->comment_balloon )
+			    withdraw_balloon( menu->comment_balloon );
 			XPutBackEvent( dpy, &tmp_e );
 			return ;    /* pointer has moved into other window - ignore this event! */
 		}
+		if( menu->comment_balloon && !get_flags( menu->state, AS_MenuBalloonShown)) 
+		{
+   			set_flags( menu->state, AS_MenuBalloonShown);
+		    display_balloon( menu->comment_balloon );
+		}
 		/* must get current pointer position as MotionNotify events
 		   tend to accumulate while we are drawing and we start getting
 		   late with menu selection, creating an illusion of slowness */
@@ -1091,16 +1286,7 @@ on_menu_pointer_event( ASInternalWindow *asiw, ASEvent *event )
 				selection = py/menu->item_height ;
 			selection += menu->top_item ;
 
-            if( selection != menu->selected_item )
-            {
-                if( xmev->state & ButtonAnyMask )
-                    press_menu_item( menu, selection, False );
-                else
-                    select_menu_item( menu, selection );
-            }
-            if( px > canvas->width - ( menu->arrow_space + DEFAULT_MENU_SPACING ) &&
-                menu->submenu == NULL )
-                run_item_submenu( menu, selection );
+			handle_menu_selection( menu, xmev->state, px, selection, True);
         }
     }
 }
@@ -1172,14 +1358,14 @@ on_menu_keyboard_event( ASInternalWindow *asiw, ASEvent *event )
 				case XK_k:
 				case XK_p:
 					if( menu->selected_item > 0 )
-	                	select_menu_item( menu, menu->selected_item-1 );
+	                	select_menu_item( menu, menu->selected_item-1, True );
                     break;
 				case XK_Tab :
 				case XK_Down:
 				case XK_n:
 				case XK_j:
 					if( menu->selected_item < menu->items_num )
-	                	select_menu_item( menu, menu->selected_item+1 );
+	                	select_menu_item( menu, menu->selected_item+1, True );
                     break;
 				case XK_Right:
 				case XK_Return :
@@ -1204,7 +1390,9 @@ on_menu_look_feel_changed( ASInternalWindow *asiw, ASFeel *feel, MyLook *look, A
     {
 		ASHints *hints ;
 		unsigned int tbar_width ;
+		MenuData *md ;
 
+   /*      if( get_flags( what, FEEL_CONFIG ) )       */
         if( menu->items )
         {
             register int i = menu->items_num;
@@ -1215,16 +1403,13 @@ on_menu_look_feel_changed( ASInternalWindow *asiw, ASFeel *feel, MyLook *look, A
 			menu->items_num = 0 ;
         }
 
-        if( get_flags( what, FEEL_CONFIG ) )
+        md = FindPopup (menu->name, False);
+        if( md == NULL )
         {
-            MenuData *md = FindPopup (menu->name, False);
-            if( md == NULL )
-            {
-                menu_destroy( asiw );
-                return ;
-            }
-            set_asmenu_data( menu, md, False );
+        	menu_destroy( asiw );
+            return ;
         }
+        set_asmenu_data( menu, md, False, get_flags(look->flags, MenuShowUnavailable), feel->recent_submenu_items );
         set_asmenu_look( menu, look );
 		hints = make_menu_hints( menu );
 	    estimate_titlebar_size( hints, &tbar_width, NULL );
@@ -1284,7 +1469,15 @@ make_menu_hints( ASMenu *menu )
 	hints = safecalloc( 1, sizeof(ASHints) );
 
 	/* normal hints : */
-    hints->names[0] = mystrdup(menu->title?menu->title:menu->name);
+	if( menu->title ) 
+	{
+    	hints->names[0] = mystrdup(menu->title);
+	    hints->names_encoding[0] = get_flags( menu->state, AS_MenuTitleIsUTF8 )? AS_Text_UTF8:AS_Text_ASCII;
+	}else
+	{
+    	hints->names[0] = mystrdup(menu->name);
+	    hints->names_encoding[0] = get_flags( menu->state, AS_MenuNameIsUTF8 )? AS_Text_UTF8:AS_Text_ASCII;
+	}
     hints->names[1] = mystrdup(ASMENU_RES_CLASS);
     /* these are merely shortcuts to the above list DON'T FREE THEM !!! */
     hints->res_name  = hints->names[1];
@@ -1342,6 +1535,8 @@ show_asmenu( ASMenu *menu, int x, int y )
     static char *ASMenuStyleNames[2] = {"ASMenu",NULL} ;
     ASDatabaseRecord *db_rec;
 
+	LOCAL_DEBUG_OUT( "menu(%s) - encoding set in hints is %d", hints->names[0], hints->names_encoding[0] ); 
+
     asiw->data = (ASMagic*)menu;
 
     asiw->register_subwindows = menu_register_subwindows;
@@ -1349,6 +1544,7 @@ show_asmenu( ASMenu *menu, int x, int y )
     asiw->on_hilite_changed = on_menu_hilite_changed ;
     asiw->on_pressure_changed = on_menu_pressure_changed;
     asiw->on_pointer_event = on_menu_pointer_event;
+	asiw->on_scroll_event = on_menu_scroll_event;
     asiw->on_keyboard_event = on_menu_keyboard_event;
     asiw->on_look_feel_changed = on_menu_look_feel_changed;
 	asiw->on_root_background_changed = on_menu_root_background_changed ;
@@ -1428,6 +1624,7 @@ show_asmenu( ASMenu *menu, int x, int y )
 		destroy_asdb_record( db_rec, False );
     }
     /* lets make sure we got everything right : */
+	LOCAL_DEBUG_OUT( "menu(%s) - encoding set in hints is %d", hints->names[0], hints->names_encoding[0] ); 
     check_hints_sanity (ASDefaultScr, hints, &status, menu->main_canvas->w );
     check_status_sanity (ASDefaultScr, &status);
 
@@ -1445,6 +1642,8 @@ show_asmenu( ASMenu *menu, int x, int y )
         }
     }
 #endif
+	LOCAL_DEBUG_OUT( "menu(%s) - encoding set in hints is %d", hints->names[0], hints->names_encoding[0] );
+
     menu->owner = AddInternalWindow( menu->main_canvas->w, &asiw, &hints, &status );
 
     /* need to cleanup if we failed : */
@@ -1468,8 +1667,8 @@ run_submenu( ASMenu *supermenu, MenuData *md, int x, int y )
     if( md )
     {
 		menu = create_asmenu(md->name);
-        menu->rendered = False ;
-		set_asmenu_data( menu, md, True );
+		clear_flags( menu->state, AS_MenuRendered);
+		set_asmenu_data( menu, md, True, get_flags(Scr.Look.flags, MenuShowUnavailable), Scr.Feel.recent_submenu_items );
         set_asmenu_look( menu, &Scr.Look );
         /* will set scroll position when ConfigureNotify arrives */
         menu->supermenu = supermenu;
@@ -1480,26 +1679,25 @@ run_submenu( ASMenu *supermenu, MenuData *md, int x, int y )
 }
 
 
-void
+ASMenu *
 run_menu_data( MenuData *md )
 {
+    ASMenu *menu = NULL ;
     int x = 0, y = 0;
 	Bool persistent = (get_flags( Scr.Feel.flags, PersistentMenus ) || ASTopmostMenu == NULL );
 
     close_asmenu(&ASTopmostMenu);
 
-	if( persistent )
+	if( persistent && md != NULL )
 	{
-        if( md == NULL )
-    	    return;
-
-    	if( !ASQueryPointerRootXY(&x,&y) )
+	    if( !ASQueryPointerRootXY(&x,&y) )
     	{
         	x = (Scr.MyDisplayWidth*2)/3;
         	y = (Scr.MyDisplayHeight*3)/ 4;
-    	}
+	    }
     	ASTopmostMenu = run_submenu(NULL, md, x, y );
 	}
+    return menu;
 }
 
 void
@@ -1533,7 +1731,7 @@ ASMenu *
 find_topmost_transient_menu( ASMenu *menu )
 {
     if( menu && menu->supermenu && menu->supermenu->magic == MAGIC_ASMENU )
-        if( !menu->supermenu->pinned )
+        if( !get_flags(menu->supermenu->state, AS_MenuPinned) )
             return find_topmost_transient_menu( menu->supermenu );
     return menu;
 }
@@ -1555,7 +1753,7 @@ pin_asmenu( ASMenu *menu )
         if( menu_to_close != menu )
             close_asmenu( &menu_to_close );
 
-        menu->pinned = True ;
+		set_flags(menu->state, AS_MenuPinned);
         if( menu->owner )
         {
             clear_flags( menu->owner->hints->function_mask, AS_FuncPinMenu);
@@ -1572,7 +1770,7 @@ Bool
 is_menu_pinnable( ASMenu *menu )
 {
     if( menu && menu->magic == MAGIC_ASMENU )
-        return !(menu->pinned);
+        return !get_flags(menu->state, AS_MenuPinned);
     return False;
 }
 
