@@ -30,6 +30,8 @@
 
 #include "asinternals.h"
 
+#include <stdlib.h>
+#include <unistd.h>
 #include "../../libAfterStep/wmprops.h"
 #include "../../libAfterStep/session.h"
 #include "../../libAfterStep/moveresize.h"
@@ -238,7 +240,7 @@ LOCAL_DEBUG_CALLER_OUT( "new(%+d%+d), old(%+d%+d), max(%+d,%+d)", newx, newy, Sc
         /* traverse window list and redo the titlebar/buttons if necessary */
         iterate_asbidirlist( Scr.Windows->clients, viewport_aswindow_iter_func, NULL, NULL, False );
         /* TODO: autoplace sticky icons so they don't wind up over a stationary icon */
-	    check_screen_panframes(&Scr);
+	    check_screen_panframes(ASDefaultScr);
 		if (grab)
 			ungrab_server();
     }
@@ -339,7 +341,7 @@ LOCAL_DEBUG_CALLER_OUT( "new(%d%+d%+d), old(%d%+d%+d), max(%+d,%+d)", new_desk,
     if( old_desk != new_desk && IsValidDesk( old_desk ) )
     {
         int client_count = 0 ;
-	    if( get_flags( AfterStepState, ASS_NormalOperation) )
+	    if( get_flags( AfterStepState, ASS_NormalOperation) && get_flags( Scr.Feel.flags, AnimateDeskChange))
 		{	
 			cover_desktop();
 			desk_covered = True ;
@@ -406,7 +408,7 @@ LOCAL_DEBUG_CALLER_OUT( "new(%d%+d%+d), old(%d%+d%+d), max(%+d,%+d)", new_desk,
         /* traverse window list and redo the titlebar/buttons if necessary */
         iterate_asbidirlist( Scr.Windows->clients, deskviewport_aswindow_iter_func, desk_id.ptr, NULL, False );
         /* TODO: autoplace sticky icons so they don't wind up over a stationary icon */
-	    check_screen_panframes(&Scr);
+	    check_screen_panframes(ASDefaultScr);
 		if ( force_grab)
 			ungrab_server();
     }
@@ -426,7 +428,7 @@ LOCAL_DEBUG_CALLER_OUT( "new(%d%+d%+d), old(%d%+d%+d), max(%+d,%+d)", new_desk,
                 	if( circ_list[i] != NULL && circ_list[i]->magic == MAGIC_ASWINDOW )
                     	if( ASWIN_DESK(circ_list[i]) == new_desk )
                     	{
-                        	focus_aswindow( circ_list[i] );
+                        	focus_aswindow( circ_list[i], FOCUS_ASW_CAN_AUTORAISE );
                         	break;
                     	}
         	}else
@@ -469,84 +471,6 @@ void
 ChangeDesks (int new_desk)
 {
 	ChangeDeskAndViewport ( new_desk, Scr.Vx, Scr.Vy, False);
-#if 0
-    int old_desk = Scr.CurrentDesk ;
-LOCAL_DEBUG_CALLER_OUT( "new_desk(%d)->old_desk(%d)", new_desk, old_desk );
-    if( Scr.CurrentDesk == new_desk )
-        return;
-
-    if( IsValidDesk( old_desk ) )
-		Scr.LastValidDesk = old_desk ;
-
-    /* we have to handle all the pending ConfigureNotifys here : */
-    ConfigureNotifyLoop();
-
-    if( IsValidDesk( Scr.CurrentDesk ) )
-    {
-        int client_count = 0 ;
-        iterate_asbidirlist( Scr.Windows->clients, count_desk_client_iter_func, (void*)&client_count, NULL, False );
-        if( client_count != 0 )
-            old_desk = INVALID_DESK ;
-    }
-
-    Scr.CurrentDesk = new_desk;
-    if( as_desk2ext_desk (Scr.wmprops, new_desk) == INVALID_DESKTOP_PROP )
-        set_desktop_num_prop( Scr.wmprops, new_desk, Scr.Root, True );
-
-    set_current_desk_prop ( Scr.wmprops, new_desk);
-
-    /* SendPacket( -1, M_NEW_DESK, 1, new_desk); */
-	/* yeld to let modules handle desktop change */
-	sleep_a_millisec(500);
-    /* Scan the window list, mapping windows on the new Desk, unmapping
-	 * windows on the old Desk; do this in reverse order to reduce client
-	 * expose events */
-	grab_server();
-    iterate_asbidirlist( Scr.Windows->clients, change_aswindow_desk_iter_func, (void*)new_desk, NULL, False );
-    ungrab_server();
-
-    if (get_flags(Scr.Feel.flags, ClickToFocus))
-	{
-        int i ;
-        int circ_count = VECTOR_USED(*(Scr.Windows->circulate_list));
-        if( !get_flags( Scr.Feel.flags, DontRestoreFocus ) )
-        {
-            ASWindow **circ_list = VECTOR_HEAD(ASWindow*,*(Scr.Windows->circulate_list));
-            for( i =0 ; i < circ_count ; ++i )
-                if( circ_list[i] != NULL && circ_list[i]->magic == MAGIC_ASWINDOW )
-                    if( ASWIN_DESK(circ_list[i]) == new_desk )
-                    {
-                        focus_aswindow( circ_list[i] );
-                        break;
-                    }
-        }else
-            i = circ_count ;
-
-        if( i >= circ_count )
-            hide_focus();
-    }
-
-    if( IsValidDesk(new_desk) )
-        restack_window_list( new_desk, False );
-
-    /* Change the look to this desktop's one if it really changed */
-#ifdef DIFFERENTLOOKNFEELFOREACHDESKTOP
-	QuickRestart ("look&feel");
-#endif
-
-    /* we need to set the desktop background : */
-	if( (IsValidDesk( new_desk ) && IsValidDesk( old_desk )) ||
-		Scr.LastValidDesk != new_desk )
-	{
-	    change_desktop_background(new_desk);
-	}/* otherwise we were switching to service desk number in order to switch
-		viewport and desktop change is not needed */
-
-#if 0
-    /* autoplace sticky icons so they don't wind up over a stationary icon */
-	AutoPlaceStickyIcons ();
-#endif
-#endif
 
 }
 
@@ -718,11 +642,19 @@ release_old_background( int desk, Bool forget )
         return ;
 	
 	imname = make_myback_image_name( &(Scr.Look), back->name );
+#if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
+	print_ashash( Scr.image_manager->image_hash, string_print );
+#endif
 
     if( back->loaded_im_name )
     {
+#if 0
         im = query_asimage( Scr.image_manager, back->data );
         LOCAL_DEBUG_OUT( "query_asimage \"%s\" - returned %p", back->data?back->data:"NULL", im );
+#else		
+        im = query_asimage( Scr.image_manager, back->loaded_im_name );
+        LOCAL_DEBUG_OUT( "query_asimage \"%s\" - returned %p", back->loaded_im_name, im );
+#endif		
     }
 	
 	if( im == NULL ) 
@@ -749,17 +681,22 @@ release_old_background( int desk, Bool forget )
 #endif
     if( im != NULL )
     {
+        LOCAL_DEBUG_OUT( "im = %p, ref_count = %d", im, im->ref_count );
+
         if( Scr.RootImage == im )
         {                                      /* always do that ! */
-            safe_asimage_destroy( im );
             Scr.RootImage = NULL ;
-        }
+            if( safe_asimage_destroy( im ) <= 0 )
+				im = NULL ;
+        } 
         
-		if( forget || im->width*im->height >= Scr.Look.KillBackgroundThreshold )
+		LOCAL_DEBUG_OUT( "im = %p, ref_count = %d", im, im?im->ref_count:0 );
+		
+		if( im != NULL && (forget || im->width*im->height >= Scr.Look.KillBackgroundThreshold) )
         {
-            LOCAL_DEBUG_OUT( "im = %p, ref_count = %d", im, im->ref_count );
-            safe_asimage_destroy( im );
-        }else if( strcmp( im->name, imname) != 0 )	 /* we need to store it for future use ! */
+            if( safe_asimage_destroy( im ) <= 0 )
+				im = NULL ;
+        }else if( im != NULL && strcmp( im->name, imname) != 0 )	 /* we need to store it for future use ! */
 		{	                                     	 /* for future references we store image under imname */
 			if( im->ref_count > 1 )
 			{ 		   /* then something else is using this image, and we need to duplicate it  */		   
@@ -770,10 +707,14 @@ release_old_background( int desk, Bool forget )
 			store_asimage( Scr.image_manager, im, imname );
 		}	 
 		if( back->loaded_im_name )
-		{	
+		{
 			free( back->loaded_im_name );
+			back->loaded_im_name = NULL ; 
+		}
+		if( im )
+		{	
 			back->loaded_im_name = mystrdup(im->name); 
-			LOCAL_DEBUG_OUT( "loaded_im_name = \"%s\"", back->loaded_im_name );
+            LOCAL_DEBUG_OUT( "im = %p, ref_count = %d, loaded_im_name = \"%s\"", im, im->ref_count, back->loaded_im_name );
 		}
     }
 	free( imname );
@@ -792,6 +733,18 @@ release_old_background( int desk, Bool forget )
 	}	 
 }
 
+void
+release_all_old_background( Bool forget )
+{
+	LOCAL_DEBUG_OUT( "as_desk_numbers = %p, as_desk_num = %ld", Scr.wmprops->as_desk_numbers, Scr.wmprops->as_desk_num );
+	if( Scr.wmprops->as_desk_numbers != NULL )
+	{
+		int i = Scr.wmprops->as_desk_num ;
+		while( --i >= 0 ) 
+			release_old_background( Scr.wmprops->as_desk_numbers[i], forget );		 
+	}
+}
+
 ASImage*
 make_desktop_image( int desk, MyBackground *new_back )
 {
@@ -867,7 +820,7 @@ make_desktop_image( int desk, MyBackground *new_back )
 
 	if( new_im != NULL ) 
 	{
-		set_string_value( &(new_back->loaded_im_name), mystrdup(new_im->name), NULL, 0 );
+		set_string( &(new_back->loaded_im_name), mystrdup(new_im->name) );
 		LOCAL_DEBUG_OUT( "produced image %p", new_im );
 	}
 	
@@ -995,7 +948,8 @@ start_background_xfer( ASImage *new_im )
 		/* under Windows XImage must not exceed 32K , or performance drops sagnificantly */
 		data->lines_per_iteration = 7800/new_im->width ;
 #else		
-		data->lines_per_iteration = Scr.MyDisplayHeight/90;
+		data->lines_per_iteration = min(new_im->height,Scr.MyDisplayHeight/
+					(Scr.Feel.desk_cover_animation_steps==0?1:Scr.Feel.desk_cover_animation_steps));
 #endif
 	}
 	if( data->lines_per_iteration == 0 ) 
@@ -1011,7 +965,9 @@ start_background_xfer( ASImage *new_im )
 	}
 #endif	   
 
-	do_background_xfer_iter( data );
+	timer_new (100, do_background_xfer_iter, data);	
+
+//	do_background_xfer_iter( data );
 	return data;
 }
 
@@ -1074,6 +1030,7 @@ LOCAL_DEBUG_CALLER_OUT( "%p:\"%s\", pmap %lX ", vdata, data->im_name, data->targ
         	XSetWindowBackgroundPixmap( dpy, Scr.Root, Scr.RootBackground->pmap );
         	XClearWindow( dpy, Scr.Root );
         	set_xrootpmap_id (Scr.wmprops, Scr.RootBackground->pmap );
+			set_as_background(Scr.wmprops, Scr.RootBackground->pmap );
 			ASSync(False);
 		}
 	}	 
@@ -1137,6 +1094,8 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
     }
 	display_progress( True, "Releasing old background ..."); 
     release_old_background( old_desk, (desk==old_desk) );
+	display_progress( False, "Done."); 
+
 	if( new_back->loaded_pixmap ) 
 	{
 		ASBackgroundHandler *bh = Scr.RootBackground ;			
@@ -1160,7 +1119,9 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
 			display_progress( True, "Setting background to previously generated pixmap ..."); 
 			XSetWindowBackgroundPixmap( dpy, Scr.Root, new_back->loaded_pixmap );
 			XClearWindow( dpy, Scr.Root );
+			display_progress( False, "Done."); 
 			set_xrootpmap_id (Scr.wmprops, new_back->loaded_pixmap );
+			set_as_background(Scr.wmprops, new_back->loaded_pixmap );
 			display_progress( True, "Done with background change. It may take a moment for X server to display it."); 
 			ASSync(False);
 			old_desk = desk ;
@@ -1168,7 +1129,7 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
 		}
 	}		  
 
-   	display_progress( True, "Changing background for desktop #%d ...", desk);
+   	display_progress( True, "Changing background for desktop #%d :", desk);
 	
 
     if( new_back->type == MB_BackCmd )
@@ -1177,7 +1138,7 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
     }else
         new_im = make_desktop_image( desk, new_back );
 
-	display_progress( True, "Background image generated."); 
+	display_progress( True, "     Background image generated."); 
     LOCAL_DEBUG_OUT( "im(%p)", new_im );
     if( new_im )
     {
@@ -1204,7 +1165,10 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
             new_im->height != bh->pmap_height) )
         {
 			if( Scr.wmprops->root_pixmap == bh->pmap )
+			{	
 				set_xrootpmap_id (Scr.wmprops, None );
+				set_as_background(Scr.wmprops, None );
+			}
             XFreePixmap( dpy, bh->pmap );
             ASSync(False);
             LOCAL_DEBUG_OUT( "root pixmap with id %lX destroyed", bh->pmap );
@@ -1257,7 +1221,8 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
 			/* can't animate if pixmap is tiled - X is slow then */
 			!get_flags( Scr.Feel.flags, DontAnimateBackground) && 
 			/* when we do this for the first time  - we better do it all at once */
-			Scr.wmprops->root_pixmap != None )
+			Scr.wmprops->root_pixmap != None 
+		   )
 		{	
 			Bool tiled = ( new_im->width < Scr.MyDisplayWidth || new_im->height < Scr.MyDisplayHeight );
 			if( old_pmap != bh->pmap )
@@ -1271,32 +1236,46 @@ LOCAL_DEBUG_CALLER_OUT( "desk(%d)->old_desk(%d)->new_back(%p)->old_back(%p)", de
 			}
 			if( !already_transferring )
 			{	
-		 		display_progress( True, "Animating background change for better responsiveness ...");
+		 		display_progress( True, "     Animating background change for better responsiveness ...");
+//			    remove_desktop_cover();
 				start_background_xfer( new_im );  /* we need to do it in small steps! */
+				display_progress( False, "Done."); 
 			}
 		}else
 		{	
+			int pid = -1 ;
 			if( !already_transferring )
 			{	
-        		if( !asimage2drawable( Scr.asv, bh->pmap, new_im, Scr.DrawGC, 0, 0, 0, 0, new_im->width, new_im->height, True) )
+       			if( !asimage2drawable( Scr.asv, bh->pmap, new_im, Scr.DrawGC, 0, 0, 0, 0, new_im->width, new_im->height, True) )
 					show_warning( "failed to draw root background onto pixmap");
         		flush_asimage_cache(new_im);
 			}
-        	XSetWindowBackgroundPixmap( dpy, Scr.Root, bh->pmap );
-        	XClearWindow( dpy, Scr.Root );
+			if( pid <= 0 )
+			{
+        		XSetWindowBackgroundPixmap( dpy, Scr.Root, bh->pmap );
+        		XClearWindow( dpy, Scr.Root );
+				if( pid == 0 ) 
+					_exit(0);
+			}
         	set_xrootpmap_id (Scr.wmprops, bh->pmap );
+			set_as_background(Scr.wmprops, bh->pmap );
 		}  
 		if( Scr.RootImage == NULL )
 			Scr.RootImage = dup_asimage(Scr.RootBackground->im) ;
     }else
+	{	
         set_xrootpmap_id (Scr.wmprops, None );
+		set_as_background(Scr.wmprops, None );
+	}
     ASSync(False);
 #ifdef DEBUG_ALLOCS	
     print_asimage_registry();
 	spool_unfreed_mem (__FUNCTION__, "");
 #endif	
     LOCAL_DEBUG_OUT("done%s","" );
-    remove_desktop_cover();
+	display_progress( True, "Done changing background."); 
+
+//    remove_desktop_cover();
 }
 
 /*************************************************************************
@@ -1364,7 +1343,7 @@ HandleBackgroundRequest( ASEvent *event )
     Bool res = False;
     Bool do_tint = True ;
 
-    if( xcli->data.s[6] == xcli->data.s[7] == xcli->data.s[8] )
+    if( xcli->data.s[6] == xcli->data.s[7] && xcli->data.s[6] == xcli->data.s[8] )
         do_tint = !(xcli->data.s[6] == 0 || xcli->data.s[6] == 0x7F );
 
     LOCAL_DEBUG_OUT("pmap(%lX/%lu)->clip_pos(%+d%+d)->clip_size(%dx%d)->scale(%d)->tint(%lX)->window(%lX)", p, p, clip_x, clip_y, clip_width, clip_height, flags, tint, event->x.xclient.window );
